=== modified file 'bzrlib/conflicts.py'
--- bzrlib/conflicts.py	2006-04-05 02:33:03 +0000
+++ bzrlib/conflicts.py	2006-04-05 04:29:35 +0000
@@ -225,28 +225,39 @@
     def __ne__(self, other):
         return not self.__eq__(other)
 
+    def __str__(self):
+        return self.format % self.__dict__
+
     @staticmethod
     def factory(type, **kwargs):
         global ctype
         return ctype[type](**kwargs)
 
-class ContentsConflict(Conflict):
-    typestring = 'contents conflict'
-
-class TextConflict(ContentsConflict):
-    typestring = 'text conflict'
 
 class PathConflict(Conflict):
     typestring = 'path conflict'
-    def __init__(self, path, conflict_path, file_id=None):
+    format = 'Path conflict: %(path)s / %(conflict_path)s'
+    def __init__(self, path, conflict_path=None, file_id=None):
         Conflict.__init__(self, path, file_id)
         self.conflict_path = conflict_path
 
     def as_stanza(self):
         s = Conflict.as_stanza(self)
-        s.add('conflict_path', self.conflict_path)
+        if self.conflict_path is not None:
+            s.add('conflict_path', self.conflict_path)
         return s
 
+
+class ContentsConflict(PathConflict):
+    typestring = 'contents conflict'
+    format = 'Contents conflict in %(path)s'
+
+
+class TextConflict(PathConflict):
+    typestring = 'text conflict'
+    format = 'Text conflict in %(path)s'
+
+
 class HandledConflict(Conflict):
     def __init__(self, action, path, file_id=None):
         Conflict.__init__(self, path, file_id)
@@ -257,6 +268,7 @@
         s.add('action', self.action)
         return s
 
+
 class HandledPathConflict(HandledConflict):
     def __init__(self, action, path, conflict_path, file_id=None,
                  conflict_file_id=None):
@@ -271,28 +283,43 @@
             s.add('conflict_file_id', self.conflict_file_id)
             
         return s
-    
+
+
 class DuplicateID(HandledPathConflict):
     typestring = 'duplicate id'
+    format = 'Conflict adding id to %(conflict_path)s.  %(action)s %(path)s.'
+
 
 class DuplicateEntry(HandledPathConflict):
     typestring = 'duplicate'
+    format = 'Conflict adding file %(conflict_path)s.  %(action)s %(path)s.'
+
 
 class ParentLoop(HandledPathConflict):
     typestring = 'parent loop'
+    format = 'Conflict moving %(conflict_path)s into %(path)s.  %(action)s.'
+
 
 class UnversionedParent(HandledConflict):
     typestring = 'unversioned parent'
+    format = 'Conflict adding versioned files to %(path)s.  %(action)s.'
+
 
 class MissingParent(HandledConflict):
     typestring = 'missing parent'
+    format = 'Conflict adding files to %(path)s.  %(action)s.'
+
+
 
 ctype = {}
+
+
 def register_types(*conflict_types):
     """Register a Conflict subclass for serialization purposes"""
     global ctype
     for conflict_type in conflict_types:
         ctype[conflict_type.typestring] = conflict_type
 
+
 register_types(ContentsConflict, TextConflict, PathConflict, DuplicateID,
-               DuplicateEntry, ParentLoop, UnversionedParent, MissingParent)
+               DuplicateEntry, ParentLoop, UnversionedParent, MissingParent,)

=== modified file 'bzrlib/merge.py'
--- bzrlib/merge.py	2006-04-05 02:33:03 +0000
+++ bzrlib/merge.py	2006-04-05 04:29:35 +0000
@@ -749,7 +749,7 @@
             else:
                 this_path = "<deleted>"
             file_id = self.tt.final_file_id(trans_id)
-            c = Conflict.factory(conflict_type, path=this_path,
+            c = Conflict.factory('path conflict', path=this_path,
                                  conflict_path=other_path, file_id=file_id)
             self.cooked_conflicts.append(c)
 

=== modified file 'bzrlib/tests/test_conflicts.py'
--- bzrlib/tests/test_conflicts.py	2006-04-05 02:33:03 +0000
+++ bzrlib/tests/test_conflicts.py	2006-04-05 04:29:35 +0000
@@ -54,8 +54,8 @@
         self.assertEqual(len(list(tree.list_files())), 6)
         conflicts = list(tree.conflict_lines())
         self.assertEqual(len(conflicts), 2)
-        self.assert_('hello' in conflicts[0][2])
-        self.assert_('hello.sploo' in conflicts[1][2])
+        self.assert_('hello' in conflicts[0].path)
+        self.assert_('hello.sploo' in conflicts[1].path)
         restore('hello')
         restore('hello.sploo')
         self.assertEqual(len(list(tree.conflict_lines())), 0)

=== modified file 'bzrlib/tests/test_merge_core.py'
--- bzrlib/tests/test_merge_core.py	2006-04-05 01:02:06 +0000
+++ bzrlib/tests/test_merge_core.py	2006-04-05 04:29:35 +0000
@@ -6,6 +6,7 @@
 import bzrlib
 from bzrlib.add import smart_add_tree
 from bzrlib.builtins import merge
+from bzrlib.conflicts import ContentsConflict, TextConflict, PathConflict
 from bzrlib.errors import (NotBranchError, NotVersionedError,
                            WorkingTreeNotRevision, BzrCommandError, NoDiff3)
 from bzrlib.inventory import RootEntry
@@ -191,7 +192,7 @@
         builder.add_file("1", "TREE_ROOT", "name1", "hello1", False)
         builder.change_name("1", other="name2", this="name3")
         conflicts = builder.merge()
-        self.assertEqual(conflicts, [('path conflict', '1', 'name3', 'name2')])
+        self.assertEqual(conflicts, [PathConflict('name3', 'name2', '1')])
         builder.cleanup()
 
     def test_merge_one(self):
@@ -227,7 +228,7 @@
         conflicts = builder.merge()
         path2 = pathjoin('dir2', 'file1')
         path3 = pathjoin('dir3', 'file1')
-        self.assertEqual(conflicts, [('path conflict', '4', path3, path2)])
+        self.assertEqual(conflicts, [PathConflict(path3, path2, '4')])
         builder.cleanup()
 
     def test_contents_merge(self):
@@ -278,7 +279,7 @@
         builder.add_file("1", "TREE_ROOT", "name1", "text1", True)
         builder.change_contents("1", other="text4", this="text3")
         conflicts = builder.merge(merge_factory)
-        self.assertEqual(conflicts, [('text conflict', '1', 'name1')])
+        self.assertEqual(conflicts, [TextConflict('name1', file_id='1')])
         builder.cleanup()
 
     def test_symlink_conflicts(self):
@@ -287,7 +288,8 @@
             builder.add_symlink("2", "TREE_ROOT", "name2", "target1")
             builder.change_target("2", other="target4", base="text3")
             conflicts = builder.merge()
-            self.assertEqual(conflicts, [('contents conflict', '2', 'name2')])
+            self.assertEqual(conflicts, [ContentsConflict('name2', 
+                                                          file_id='2')])
             builder.cleanup()
 
     def test_symlink_merge(self):

=== modified file 'bzrlib/tests/test_transform.py'
--- bzrlib/tests/test_transform.py	2006-02-24 19:00:42 +0000
+++ bzrlib/tests/test_transform.py	2006-04-05 04:29:35 +0000
@@ -389,22 +389,26 @@
         conflicts.apply()
 
     def test_cook_conflicts(self):
+        from bzrlib.conflicts import (DuplicateEntry, DuplicateID, 
+                                      MissingParent, UnversionedParent,
+                                      ParentLoop)
         tt, emerald, oz, old_dorothy, new_dorothy = self.get_conflicted()
         raw_conflicts = resolve_conflicts(tt)
         cooked_conflicts = cook_conflicts(raw_conflicts, tt)
-        duplicate = ('duplicate', 'Moved existing file to', 'dorothy.moved', 
-                     None, 'dorothy', 'dorothy-id')
+        duplicate = DuplicateEntry('Moved existing file to', 'dorothy.moved', 
+                                   'dorothy', None, 'dorothy-id')
         self.assertEqual(cooked_conflicts[0], duplicate)
-        duplicate_id = ('duplicate id', 'Unversioned existing file', 
-                        'dorothy.moved', None, 'dorothy', 'dorothy-id')
+        duplicate_id = DuplicateID('Unversioned existing file', 
+                                   'dorothy.moved', 'dorothy', None,
+                                   'dorothy-id')
         self.assertEqual(cooked_conflicts[1], duplicate_id)
-        missing_parent = ('missing parent', 'Not deleting', 'oz', 'oz-id')
+        missing_parent = MissingParent('Not deleting', 'oz', 'oz-id')
         self.assertEqual(cooked_conflicts[2], missing_parent)
-        unversioned_parent = ('unversioned parent', 
-                              'Versioned directory', 'oz', 'oz-id')
+        unversioned_parent = UnversionedParent('Versioned directory', 'oz',
+                                               'oz-id')
         self.assertEqual(cooked_conflicts[3], unversioned_parent)
-        parent_loop = ('parent loop', 'Cancelled move', 'oz/emeraldcity', 
-                       'emerald-id', 'oz/emeraldcity', 'emerald-id')
+        parent_loop = ParentLoop('Cancelled move', 'oz/emeraldcity', 
+                                 'oz/emeraldcity', 'emerald-id', 'emerald-id')
         self.assertEqual(cooked_conflicts[4], parent_loop)
         self.assertEqual(len(cooked_conflicts), 5)
         tt.finalize()

=== modified file 'bzrlib/tests/test_workingtree.py'
--- bzrlib/tests/test_workingtree.py	2006-04-05 01:02:06 +0000
+++ bzrlib/tests/test_workingtree.py	2006-04-05 04:29:35 +0000
@@ -22,6 +22,7 @@
 from bzrlib.branch import Branch
 import bzrlib.bzrdir as bzrdir
 from bzrlib.bzrdir import BzrDir
+from bzrlib.conflicts import *
 import bzrlib.errors as errors
 from bzrlib.errors import NotBranchError, NotVersionedError
 from bzrlib.lockdir import LockDir
@@ -213,18 +214,18 @@
         self.assertRaises(errors.UnsupportedOperation, tree.set_conflict_lines,
                           None)
         file('lala.BASE', 'wb').write('labase')
-        expected = ('contents conflict', None, 'lala')
+        expected = ContentsConflict('lala')
         self.assertEqual(list(tree.conflict_lines()), [expected])
         file('lala', 'wb').write('la')
         tree.add('lala', 'lala-id')
-        expected = ('contents conflict', 'lala-id', 'lala')
+        expected = ContentsConflict('lala', file_id='lala-id')
         self.assertEqual(list(tree.conflict_lines()), [expected])
         file('lala.THIS', 'wb').write('lathis')
         file('lala.OTHER', 'wb').write('laother')
         # When "text conflict"s happen, stem, THIS and OTHER are text
-        expected = ('text conflict', 'lala-id', 'lala')
+        expected = TextConflict('lala', file_id='lala-id')
         self.assertEqual(list(tree.conflict_lines()), [expected])
         os.unlink('lala.OTHER')
         os.mkdir('lala.OTHER')
-        expected = ('contents conflict', 'lala-id', 'lala')
+        expected = ContentsConflict('lala', file_id='lala-id')
         self.assertEqual(list(tree.conflict_lines()), [expected])

=== modified file 'bzrlib/transform.py'
--- bzrlib/transform.py	2006-04-05 02:33:03 +0000
+++ bzrlib/transform.py	2006-04-05 04:29:35 +0000
@@ -1151,12 +1151,12 @@
 def cook_conflicts(raw_conflicts, tt):
     """Generate a list of cooked conflicts, sorted by file path"""
     def key(conflict):
-        if conflict[2] is not None:
-            return conflict[2], conflict[0]
-        elif len(conflict) == 6:
-            return conflict[4], conflict[0]
+        if conflict.path is not None:
+            return conflict.path, conflict.typestring
+        elif getattr(conflict, "conflict_path", None) is not None:
+            return conflict.conflict_path, conflict.typestring
         else:
-            return None, conflict[0]
+            return None, conflict.typestring
 
     return sorted(list(iter_cook_conflicts(raw_conflicts, tt)), key=key)
 
@@ -1185,25 +1185,4 @@
 def conflicts_strings(conflicts):
     """Generate strings for the provided conflicts"""
     for conflict in conflicts:
-        conflict_type = conflict[0]
-        if conflict_type == 'text conflict':
-            yield 'Text conflict in %s' % conflict[2]
-        elif conflict_type == 'contents conflict':
-            yield 'Contents conflict in %s' % conflict[2]
-        elif conflict_type == 'path conflict':
-            yield 'Path conflict: %s / %s' % conflict[2:]
-        elif conflict_type == 'duplicate id':
-            vals = (conflict[4], conflict[1], conflict[2])
-            yield 'Conflict adding id to %s.  %s %s.' % vals
-        elif conflict_type == 'duplicate':
-            vals = (conflict[4], conflict[1], conflict[2])
-            yield 'Conflict adding file %s.  %s %s.' % vals
-        elif conflict_type == 'parent loop':
-            vals = (conflict[4], conflict[2], conflict[1])
-            yield 'Conflict moving %s into %s.  %s.' % vals
-        elif conflict_type == 'unversioned parent':
-            vals = (conflict[2], conflict[1])
-            yield 'Conflict adding versioned files to %s.  %s.' % vals
-        elif conflict_type == 'missing parent':
-            vals = (conflict[2], conflict[1])
-            yield 'Conflict adding files to %s.  %s.' % vals
+        yield str(conflict)

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2006-04-05 01:02:06 +0000
+++ bzrlib/workingtree.py	2006-04-05 04:29:35 +0000
@@ -50,7 +50,7 @@
 from bzrlib.atomicfile import AtomicFile
 from bzrlib.branch import (Branch,
                            quotefn)
-from bzrlib.conflicts import (stanza_conflicts, conflict_stanzas,
+from bzrlib.conflicts import (stanza_conflicts, conflict_stanzas, Conflict,
                               CONFLICT_SUFFIXES)
 import bzrlib.bzrdir as bzrdir
 from bzrlib.decorators import needs_read_lock, needs_write_lock
@@ -1316,7 +1316,8 @@
                         text = False
                         break
             ctype = {True: 'text conflict', False: 'contents conflict'}[text]
-            yield (ctype, self.path2id(conflicted), conflicted)
+            yield Conflict.factory(ctype, path=conflicted, 
+                                   file_id=self.path2id(conflicted))
 
 class WorkingTree3(WorkingTree):
     """This is the Format 3 working tree.

